@@ -22,66 +22,63 @@
 
 #include "notmuch-client.h"
 
-typedef int (*command_function_t) (void *ctx, int argc, char *argv[]);
+typedef int (*command_function_t) (notmuch_config_t *config, int argc, char *argv[]);
 
 typedef struct command {
     const char *name;
     command_function_t function;
-    const char *arguments;
+    notmuch_bool_t create_config;
     const char *summary;
 } command_t;
 
-#define MAX_ALIAS_SUBSTITUTIONS 3
-
-typedef struct alias {
-    const char *name;
-    const char *substitutions[MAX_ALIAS_SUBSTITUTIONS];
-} alias_t;
-
-alias_t aliases[] = {
-    { "part", { "show", "--format=raw"}},
-    { "search-tags", {"search", "--output=tags", "*"}}
-};
+static int
+notmuch_help_command (notmuch_config_t *config, int argc, char *argv[]);
 
 static int
-notmuch_help_command (void *ctx, int argc, char *argv[]);
+notmuch_command (notmuch_config_t *config, int argc, char *argv[]);
 
 static command_t commands[] = {
-    { "setup", notmuch_setup_command,
-      NULL,
+    { NULL, notmuch_command, TRUE,
+      "Notmuch main command." },
+    { "setup", notmuch_setup_command, TRUE,
       "Interactively setup notmuch for first use." },
-    { "new", notmuch_new_command,
-      "[options...]",
+    { "new", notmuch_new_command, FALSE,
       "Find and import new messages to the notmuch database." },
-    { "search", notmuch_search_command,
-      "[options...] <search-terms> [...]",
+    { "insert", notmuch_insert_command, FALSE,
+      "Add a new message into the maildir and notmuch database." },
+    { "search", notmuch_search_command, FALSE,
       "Search for messages matching the given search terms." },
-    { "show", notmuch_show_command,
-      "<search-terms> [...]",
+    { "show", notmuch_show_command, FALSE,
       "Show all messages matching the search terms." },
-    { "count", notmuch_count_command,
-      "[options...] <search-terms> [...]",
+    { "count", notmuch_count_command, FALSE,
       "Count messages matching the search terms." },
-    { "reply", notmuch_reply_command,
-      "[options...] <search-terms> [...]",
+    { "reply", notmuch_reply_command, FALSE,
       "Construct a reply template for a set of messages." },
-    { "tag", notmuch_tag_command,
-      "+<tag>|-<tag> [...] [--] <search-terms> [...]" ,
+    { "tag", notmuch_tag_command, FALSE,
       "Add/remove tags for all messages matching the search terms." },
-    { "dump", notmuch_dump_command,
-      "[<filename>] [--] [<search-terms>]",
+    { "dump", notmuch_dump_command, FALSE,
       "Create a plain-text dump of the tags for each message." },
-    { "restore", notmuch_restore_command,
-      "[--accumulate] [<filename>]",
+    { "restore", notmuch_restore_command, FALSE,
       "Restore the tags from the given dump file (see 'dump')." },
-    { "config", notmuch_config_command,
-      "[get|set] <section>.<item> [value ...]",
+    { "config", notmuch_config_command, FALSE,
       "Get or set settings in the notmuch configuration file." },
-    { "help", notmuch_help_command,
-      "[<command>]",
+    { "help", notmuch_help_command, TRUE, /* create but don't save config */
       "This message, or more detailed help for the named command." }
 };
 
+static command_t *
+find_command (const char *name)
+{
+    size_t i;
+
+    for (i = 0; i < ARRAY_SIZE (commands); i++)
+	if ((!name && !commands[i].name) ||
+	    (name && commands[i].name && strcmp (name, commands[i].name) == 0))
+	    return &commands[i];
+
+    return NULL;
+}
+
 int notmuch_format_version;
 
 static void
@@ -101,8 +98,8 @@ usage (FILE *out)
     for (i = 0; i < ARRAY_SIZE (commands); i++) {
 	command = &commands[i];
 
-	fprintf (out, "  %-11s  %s\n",
-		 command->name, command->summary);
+	if (command->name)
+	    fprintf (out, "  %-11s  %s\n", command->name, command->summary);
     }
 
     fprintf (out, "\n");
@@ -148,10 +145,9 @@ exec_man (const char *page)
 }
 
 static int
-notmuch_help_command (void *ctx, int argc, char *argv[])
+notmuch_help_command (notmuch_config_t *config, int argc, char *argv[])
 {
     command_t *command;
-    unsigned int i;
 
     argc--; argv++; /* Ignore "help" */
 
@@ -170,13 +166,10 @@ notmuch_help_command (void *ctx, int argc, char *argv[])
 	return 0;
     }
 
-    for (i = 0; i < ARRAY_SIZE (commands); i++) {
-	command = &commands[i];
-
-	if (strcmp (argv[0], command->name) == 0) {
-	    char *page = talloc_asprintf (ctx, "notmuch-%s", command->name);
-	    exec_man (page);
-	}
+    command = find_command (argv[0]);
+    if (command) {
+	char *page = talloc_asprintf (config, "notmuch-%s", command->name);
+	exec_man (page);
     }
 
     if (strcmp (argv[0], "search-terms") == 0) {
@@ -196,29 +189,23 @@ notmuch_help_command (void *ctx, int argc, char *argv[])
  * to be more clever about this in the future.
  */
 static int
-notmuch (void *ctx)
+notmuch_command (notmuch_config_t *config,
+		 unused(int argc), unused(char *argv[]))
 {
-    notmuch_config_t *config;
-    notmuch_bool_t is_new;
     char *db_path;
     struct stat st;
 
-    config = notmuch_config_open (ctx, NULL, &is_new);
-
     /* If the user has never configured notmuch, then run
      * notmuch_setup_command which will give a nice welcome message,
      * and interactively guide the user through the configuration. */
-    if (is_new) {
-	notmuch_config_close (config);
-	return notmuch_setup_command (ctx, 0, NULL);
-    }
+    if (notmuch_config_is_new (config))
+	return notmuch_setup_command (config, 0, NULL);
 
     /* Notmuch is already configured, but is there a database? */
-    db_path = talloc_asprintf (ctx, "%s/%s",
+    db_path = talloc_asprintf (config, "%s/%s",
 			       notmuch_config_get_database_path (config),
 			       ".notmuch");
     if (stat (db_path, &st)) {
-	notmuch_config_close (config);
 	if (errno != ENOENT) {
 	    fprintf (stderr, "Error looking for notmuch database at %s: %s\n",
 		     db_path, strerror (errno));
@@ -250,8 +237,6 @@ notmuch (void *ctx)
 	    notmuch_config_get_user_name (config),
 	    notmuch_config_get_user_primary_email (config));
 
-    notmuch_config_close (config);
-
     return 0;
 }
 
@@ -259,97 +244,83 @@ int
 main (int argc, char *argv[])
 {
     void *local;
+    char *talloc_report;
+    const char *command_name = NULL;
     command_t *command;
-    alias_t *alias;
-    unsigned int i, j;
-    const char **argv_local;
+    char *config_file_name = NULL;
+    notmuch_config_t *config;
+    notmuch_bool_t print_help=FALSE, print_version=FALSE;
+    int opt_index;
+    int ret = 0;
+
+    notmuch_opt_desc_t options[] = {
+	{ NOTMUCH_OPT_BOOLEAN, &print_help, "help", 'h', 0 },
+	{ NOTMUCH_OPT_BOOLEAN, &print_version, "version", 'v', 0 },
+	{ NOTMUCH_OPT_STRING, &config_file_name, "config", 'c', 0 },
+	{ 0, 0, 0, 0, 0 }
+    };
 
     talloc_enable_null_tracking ();
 
     local = talloc_new (NULL);
 
     g_mime_init (0);
+#if !GLIB_CHECK_VERSION(2, 35, 1)
     g_type_init ();
+#endif
 
     /* Globally default to the current output format version. */
     notmuch_format_version = NOTMUCH_FORMAT_CUR;
 
-    if (argc == 1)
-	return notmuch (local);
+    opt_index = parse_arguments (argc, argv, options, 1);
+    if (opt_index < 0) {
+	/* diagnostics already printed */
+	return 1;
+    }
 
-    if (strcmp (argv[1], "--help") == 0)
+    if (print_help)
 	return notmuch_help_command (NULL, argc - 1, &argv[1]);
 
-    if (strcmp (argv[1], "--version") == 0) {
+    if (print_version) {
 	printf ("notmuch " STRINGIFY(NOTMUCH_VERSION) "\n");
 	return 0;
     }
 
-    for (i = 0; i < ARRAY_SIZE (aliases); i++) {
-	alias = &aliases[i];
+    if (opt_index < argc)
+	command_name = argv[opt_index];
 
-	if (strcmp (argv[1], alias->name) == 0)
-	{
-	    int substitutions;
-
-	    argv_local = talloc_size (local, sizeof (char *) *
-				      (argc + MAX_ALIAS_SUBSTITUTIONS - 1));
-	    if (argv_local == NULL) {
-		fprintf (stderr, "Out of memory.\n");
-		return 1;
-	    }
-
-	    /* Copy all substution arguments from the alias. */
-	    argv_local[0] = argv[0];
-	    for (j = 0; j < MAX_ALIAS_SUBSTITUTIONS; j++) {
-		if (alias->substitutions[j] == NULL)
-		    break;
-		argv_local[j+1] = alias->substitutions[j];
-	    }
-	    substitutions = j;
-
-	    /* And copy all original arguments (skipping the argument
-	     * that matched the alias of course. */
-	    for (j = 2; j < (unsigned) argc; j++) {
-		argv_local[substitutions+j-1] = argv[j];
-	    }
-
-	    argc += substitutions - 1;
-	    argv = (char **) argv_local;
-	}
+    command = find_command (command_name);
+    if (!command) {
+	fprintf (stderr, "Error: Unknown command '%s' (see \"notmuch help\")\n",
+		 command_name);
+	return 1;
     }
 
-    for (i = 0; i < ARRAY_SIZE (commands); i++) {
-	command = &commands[i];
+    config = notmuch_config_open (local, config_file_name, command->create_config);
+    if (!config)
+	return 1;
 
-	if (strcmp (argv[1], command->name) == 0) {
-	    int ret;
-	    char *talloc_report;
+    ret = (command->function)(config, argc - opt_index, argv + opt_index);
 
-	    ret = (command->function)(local, argc - 1, &argv[1]);
+    notmuch_config_close (config);
 
-	    /* in the future support for this environment variable may
-	     * be supplemented or replaced by command line arguments
-	     * --leak-report and/or --leak-report-full */
+    talloc_report = getenv ("NOTMUCH_TALLOC_REPORT");
+    if (talloc_report && strcmp (talloc_report, "") != 0) {
+	/* this relies on the previous call to
+	 * talloc_enable_null_tracking
+	 */
 
-	    talloc_report = getenv ("NOTMUCH_TALLOC_REPORT");
-
-	    /* this relies on the previous call to
-	     * talloc_enable_null_tracking */
-
-	    if (talloc_report && strcmp (talloc_report, "") != 0) {
-		FILE *report = fopen (talloc_report, "w");
-		talloc_report_full (NULL, report);
-	    }
-
-	    return ret;
+	FILE *report = fopen (talloc_report, "w");
+	if (report) {
+	    talloc_report_full (NULL, report);
+	} else {
+	    ret = 1;
+	    fprintf (stderr, "ERROR: unable to write talloc log. ");
+	    perror (talloc_report);
 	}
     }
 
-    fprintf (stderr, "Error: Unknown command '%s' (see \"notmuch help\")\n",
-	     argv[1]);
-
     talloc_free (local);
 
-    return 1;
+    return ret;
 }
